var _gl;
var _canvas;
var _lastTime = 0;
var _elapsed = 0;
var _shaderProgram;
var _mvMatrix;
var _mvMatrixStack = [];
var _pMatrix;
var _eye;
var _lookAt;
var _fov = 45;
var _near = 1;
var _far = 10000;

function initWebGL(fullScreen)
{
	_canvas = document.getElementById("PsychoVision3D-Canvas");

	setWebGL();
	setShaders();

	_gl.clearColor(0.4, 0.7, 1.0, 1.0);
	_gl.enable(_gl.DEPTH_TEST);
	_gl.enable(_gl.CULL_FACE);
	_gl.blendColor(0,0,0,0);
	_gl.blendFuncSeparate(_gl.SRC_ALPHA,_gl.ONE_MINUS_SRC_ALPHA,_gl.ZERO,_gl.ONE);
	_gl.blendEquation(_gl.FUNC_ADD);
	_gl.enable(_gl.BLEND);
	_gl.cullFace(_gl.FRONT);

	_mvMatrix = new Matrix3D();
	_pMatrix = new Matrix3D();
	_eye = new Vector3D(0,0,500);
	_lookAt = new Vector3D(0,0,0);

	beginWebGL();
	mainLoop();
}

function setWebGL()
{
	try
	{
		_gl = _canvas.getContext("webgl") || _canvas.getContext("experimental-webgl");
		_gl.viewportWidth = _canvas.width;
		_gl.viewportHeight = _canvas.height;
	}
	catch (e)
	{
	}
	if (!_gl)
	{
		alert("Failed to initialize WebGL...");
	}
}

function getFragmentShader()
{
	var strFragment = "\
		precision mediump float;\
		varying vec3 v_TransformedNormal;\
		varying vec4 v_Position;\
		varying vec4 v_Color;\
		uniform vec3 u_AmbientColor;\
		uniform vec3 u_PointLightingLocation;\
		uniform vec3 u_PointLightingDiffuseColor;\
		void main(void)\
		{\
			vec3 lightDirection = normalize(u_PointLightingLocation - v_Position.xyz);\
			vec3 normal = normalize(v_TransformedNormal);\
			float diffuseLightWeighting = max(dot(normal, lightDirection), 0.0);\
			vec3 lightWeighting = u_AmbientColor\
				+ u_PointLightingDiffuseColor * diffuseLightWeighting;\
			gl_FragColor = vec4(v_Color.rgb * lightWeighting, v_Color.a);\
		}";
	var shader = _gl.createShader(_gl.FRAGMENT_SHADER);

	_gl.shaderSource(shader, strFragment);
	_gl.compileShader(shader);

	if (!_gl.getShaderParameter(shader, _gl.COMPILE_STATUS))
	{
		alert(_gl.getShaderInfoLog(shader));
		return null;
	}

   	return shader;
}

function getVertexShader()
{
	var strVertex = "\
		attribute vec3 a_VertexPosition;\
		attribute vec3 a_VertexNormal;\
		attribute vec4 a_VertexColor;\
		uniform mat4 u_MVMatrix;\
		uniform mat4 u_PMatrix;\
		uniform mat3 u_NMatrix;\
		varying vec3 v_TransformedNormal;\
		varying vec4 v_Position;\
		varying vec4 v_Color;\
		void main(void)\
		{\
			v_Position = u_MVMatrix * vec4(a_VertexPosition, 1.0);\
			gl_Position = u_PMatrix * v_Position;\
			v_TransformedNormal = u_NMatrix * a_VertexNormal;\
			v_Color = a_VertexColor;\
		}";
	var shader = _gl.createShader(_gl.VERTEX_SHADER);

	_gl.shaderSource(shader, strVertex);
	_gl.compileShader(shader);

	if (!_gl.getShaderParameter(shader, _gl.COMPILE_STATUS))
	{
		alert(_gl.getShaderInfoLog(shader));
		return null;
	}

   	return shader;
}

function setShaders()
{
	var fragmentShader = getFragmentShader();
	var vertexShader = getVertexShader();

	_shaderProgram = _gl.createProgram();
	_gl.attachShader(_shaderProgram, vertexShader);
	_gl.attachShader(_shaderProgram, fragmentShader);
	_gl.linkProgram(_shaderProgram);

	if (!_gl.getProgramParameter(_shaderProgram, _gl.LINK_STATUS))
	{
		alert("Could not initialise shaders");
	}

	_gl.useProgram(_shaderProgram);

	_shaderProgram.a_vertexPosition = _gl.getAttribLocation(_shaderProgram, "a_VertexPosition");
	_gl.enableVertexAttribArray(_shaderProgram.a_vertexPosition);
	_shaderProgram.a_vertexNormal = _gl.getAttribLocation(_shaderProgram, "a_VertexNormal");
	_gl.enableVertexAttribArray(_shaderProgram.a_vertexNormal);
	_shaderProgram.a_vertexColor = _gl.getAttribLocation(_shaderProgram, "a_VertexColor");
	_gl.enableVertexAttribArray(_shaderProgram.a_vertexColor);

	_shaderProgram.u_pMatrix = _gl.getUniformLocation(_shaderProgram, "u_PMatrix");
	_shaderProgram.u_mvMatrix = _gl.getUniformLocation(_shaderProgram, "u_MVMatrix");
	_shaderProgram.u_nMatrix = _gl.getUniformLocation(_shaderProgram, "u_NMatrix");
	_shaderProgram.u_ambientColor = _gl.getUniformLocation(_shaderProgram, "u_AmbientColor");
	_shaderProgram.u_pointLightingLocation = _gl.getUniformLocation(_shaderProgram, "u_PointLightingLocation");
	_shaderProgram.u_pointLightingDiffuseColor = _gl.getUniformLocation(_shaderProgram, "u_PointLightingDiffuseColor");

	_gl.uniform3f(_shaderProgram.u_ambientColor,0.4,0.4,0.4);
	_gl.uniform3f(_shaderProgram.u_pointLightingDiffuseColor,1,1,1);
	_gl.uniform3f(_shaderProgram.u_pointLightingLocation,5000,-5000,-10000);
}

function mvPushMatrix()
{
	var copy = _mvMatrix.copy();
	_mvMatrixStack.push(copy._e);
}

function mvPopMatrix()
{
	if (_mvMatrixStack.length == 0)
	{
		throw "Invalid mvPopMatrix!";
	}
	_mvMatrix._e = _mvMatrixStack.pop();
}

function setMatrixUniforms(pMatrix)
{
	_gl.uniformMatrix4fv(_shaderProgram.u_pMatrix, false, pMatrix);
	_gl.uniformMatrix4fv(_shaderProgram.u_mvMatrix, false, _mvMatrix._e);

	var normalMatrix = _mvMatrix.inverseMat33();
	normalMatrix = normalMatrix.transpose();
	_gl.uniformMatrix3fv(_shaderProgram.u_nMatrix, false, normalMatrix._e);
}

function degreeToRadian(degree)
{
	return degree * Math.PI / 180;
}

function radianToDegree(radian)
{
	return radian * 180 / Math.PI;
}

function beginDraw()
{
	_gl.viewport(0, 0, _gl.viewportWidth, _gl.viewportHeight);
	_gl.clear(_gl.COLOR_BUFFER_BIT | _gl.DEPTH_BUFFER_BIT);

	_pMatrix.perspective(_fov, _gl.viewportWidth / _gl.viewportHeight, _near, _far);
	_mvMatrix.identity();
}

function animate()
{
	var timeNow = new Date().getTime();
	if (_lastTime != 0)
	{
		_elapsed = (timeNow - _lastTime) / 1000.0;
	}
	_lastTime = timeNow;
}

window.requestAnimFrame = (function()
{
	return window.requestAnimationFrame ||
		window.webkitRequestAnimationFrame ||
		window.mozRequestAnimationFrame ||
		window.oRequestAnimationFrame ||
		window.msRequestAnimationFrame ||
		function(callback, element)
		{
			window.setTimeout(callback, 1000/30);
		};
})();

function mainLoop()
{
	requestAnimFrame(mainLoop);
	beginDraw();
	drawWebGL();
	animate();
}
